@@ -30,13 +30,13 @@
 #include <sys/stat.h>
 #include <string.h>
 
+#include <error.h>
 
 #include "mountlist.h"
 #include "fsusage.h"
 #include "save-cwd.h" 
 #define STREQ(s1, s2) ((strcmp (s1, s2) == 0))
 
-char *dirname ();
 void strip_trailing_slashes ();
 char *xgetcwd ();
 
@@ -123,6 +123,8 @@ static int       selected_fstype       (const char *fstype);
 static int       excluded_fstype       (const char *fstype);
 static void      exit_menu_cb          (GtkWidget  *widget,
 					gpointer   data);
+static void      about_cb              (GtkWidget  *widget,
+					gpointer   data);
 GtkWidget   **t_dial;
 
 static GnomeUIInfo file_menu[] = {
@@ -132,9 +134,20 @@ static GnomeUIInfo file_menu[] = {
   { GNOME_APP_UI_ENDOFINFO },
 };
 
+static GnomeUIInfo help_menu[] = {
+  { GNOME_APP_UI_HELP, NULL, NULL, NULL, NULL, NULL,
+    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+  { GNOME_APP_UI_ITEM, N_("About..."), NULL, about_cb, NULL, NULL,
+    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 'A', GDK_CONTROL_MASK,
+    NULL },
+  { GNOME_APP_UI_ENDOFINFO }
+};
+
 static GnomeUIInfo app_menu[] = {
   { GNOME_APP_UI_SUBTREE, N_("File"), NULL, file_menu, NULL, NULL, 
     GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+  { GNOME_APP_UI_SUBTREE, N_("Help"), NULL, help_menu, NULL, NULL,
+    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
   { GNOME_APP_UI_ENDOFINFO },
 };
 
@@ -155,6 +168,8 @@ build_app_window ()
   int                 i;
 
   window = gnome_app_new ("GDiskFree", "Disk Free");
+  gtk_signal_connect(GTK_OBJECT(window),"destroy",
+		     GTK_SIGNAL_FUNC(gtk_main_quit),NULL);
   
   w_box = gtk_vbox_new (FALSE, 0);
   i = 0;
@@ -164,6 +179,7 @@ build_app_window ()
       adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 100.0, 0.01,
 						       0.1, 0.0));
       t_dial[i] = gtk_dial_new (adjustment);
+      gtk_dial_set_view_only(GTK_DIAL(t_dial[i]), TRUE);
       gtk_widget_set_usize (t_dial[i], 75, 60);
       gtk_container_add (GTK_CONTAINER (frame), t_dial[i]);
       gtk_box_pack_start (GTK_BOX (w_box), frame, FALSE, FALSE, 0);
@@ -173,9 +189,28 @@ build_app_window ()
   }
   gnome_app_set_contents (GNOME_APP (window), w_box);
   gnome_app_create_menus (GNOME_APP (window), app_menu);
+  gtk_menu_item_right_justify (GTK_MENU_ITEM (app_menu[1].widget));
   gtk_widget_show_all (window);
 }
 
+static void
+about_cb (GtkWidget *widget, gpointer data)
+{
+  GtkWidget     *about;
+  gchar         *authors[] = {
+    "Gregory McLean <gregm@comstar.net>",
+    NULL
+  };
+
+  about = gnome_about_new ("GDiskFree", "0.1",
+			   "(C) 91, 95, 1996 Free Software Foundation, Inc.",
+			   authors,
+			   _("A quick hack to show the amount of space free "
+			     "on mounted file systems uitilizing a dial."),
+			   NULL );
+  gtk_widget_show (about);
+}
+
 static void
 print_header (void)
 {
@@ -447,6 +482,39 @@ show_disk (const char *disk)
   show_dev (disk, (char *) NULL, (char *) NULL);
 }
 
+/* Return the directory component of PATH.  If PATH doesn't have one,
+   return ".".  The return value is allocated with `g_malloc', and
+   should be released with `g_free'.  */
+static char *
+g_dirname (const char *path)
+{
+  char *newpath;
+  char *slash;
+  int length;			/* Length of result, not including NUL.  */
+
+  slash = strrchr (path, '/');
+  if (slash == 0)
+    {
+      /* File is in the current directory.  */
+      path = ".";
+      length = 1;
+    }
+  else
+    {
+      /* Remove any trailing slashes from the result.  */
+      while (slash > path && *slash == '/')
+	--slash;
+
+      length = slash - path + 1;
+    }
+  newpath = (char *) g_malloc (length + 1);
+  if (newpath == 0)
+    return 0;
+  strncpy (newpath, path, length);
+  newpath[length] = 0;
+  return newpath;
+}
+
 /* Return the root mountpoint of the filesystem on which FILE exists, in
    malloced storage.  FILE_STAT should be the result of stating FILE.  */
 static char *
@@ -473,10 +541,10 @@ find_mount_point (const char *file, const struct stat *file_stat)
       char *tmp = g_strdup (file);
       char *dir;
 
-      dir = dirname (tmp);
-      free (tmp);
+      dir = g_dirname (tmp);
+      g_free (tmp);
       rv = chdir (dir);
-      free (dir);
+      g_free (dir);
 
       if (rv < 0)
 	return NULL;
