@@ -108,14 +108,26 @@ addkey(gchar c)
 	g_free(t);
 }
 
+/* add the key's label to entry in data without borders */
+void
+addkeycbnb(GtkWidget * widget, gpointer * data)
+{
+	char *p;
+	int i;
+	gtk_label_get(GTK_LABEL(GTK_BUTTON(widget)->child),&p);
+	for(i=1;i<strlen(p)-1;i++)
+		addkey(p[i]);
+}
+
 /* add the key's label to entry in data */
 void
 addkeycb(GtkWidget * widget, gpointer * data)
 {
 	char *p;
-
+	int i;
 	gtk_label_get(GTK_LABEL(GTK_BUTTON(widget)->child),&p);
-	addkey(p[1]);
+	for(i=0;i<strlen(p);i++)
+		addkey(p[i]);
 }
 
 /*display a message in a messagebox*/
@@ -596,6 +608,14 @@ main(int argc, char *argv[])
 		{" 1 "," 2 "," 3 "," - "},
 		{" 0 "," . "," ~ "," + "}
 	};
+	
+	/*scientific*/
+	char *scientific[4][3]={
+		{" sin "," cos "," tan "},
+		{" pi "," e ", NULL},
+		{" is_complex "," Re "," Im "},
+		{" round "," sqrt ", " ! "}
+	};
 	int x,y;
 
 	bindtextdomain(PACKAGE,GNOMELOCALEDIR);
@@ -658,8 +678,14 @@ main(int argc, char *argv[])
 	for(x=0;x<4;x++) {
 		for(y=0;y<5;y++) {
 			w=gtk_button_new_with_label(numpad[y][x]);
-			gtk_signal_connect(GTK_OBJECT(w), "clicked",
-				GTK_SIGNAL_FUNC(addkeycb), NULL);
+			if(numpad[y][x][1]>='0' ||
+			   numpad[y][x][1]<='9')  {
+				gtk_signal_connect(GTK_OBJECT(w), "clicked",
+						   GTK_SIGNAL_FUNC(addkeycbnb), NULL);
+			} else {
+				gtk_signal_connect(GTK_OBJECT(w), "clicked",
+						   GTK_SIGNAL_FUNC(addkeycb), NULL);
+			}
 			gtk_table_attach_defaults(GTK_TABLE(table),w,
 				x+1,x+2,y+1,y+2);
 			gtk_widget_show(w);
@@ -679,6 +705,25 @@ main(int argc, char *argv[])
 	sciframe=gtk_frame_new(_("Scientific"));
 	gtk_box_pack_start(GTK_BOX(boxh),sciframe,FALSE,FALSE,0);
 
+	table=gtk_table_new(6,5,0);
+	gtk_table_set_col_spacings(GTK_TABLE(table),5);
+	gtk_table_set_row_spacings(GTK_TABLE(table),5);
+	for(x=0;x<3;x++) {
+		for(y=0;y<4;y++) {
+			if(scientific[y][x]) {
+				w=gtk_button_new_with_label(scientific[y][x]);
+				gtk_signal_connect(GTK_OBJECT(w), "clicked",
+						   GTK_SIGNAL_FUNC(addkeycb), NULL);
+				gtk_table_attach_defaults(GTK_TABLE(table),w,
+							  x+1,x+2,y+1,y+2);
+				gtk_widget_show(w);
+			}
+		}
+	}
+	gtk_container_add(GTK_CONTAINER(sciframe),table);
+	gtk_widget_show(table);
+	
+
 	gtk_box_pack_start(GTK_BOX(boxm),boxh,TRUE,TRUE,5);
 	gtk_widget_show(boxh);
 
