@@ -2,6 +2,7 @@
 #include <gnome.h>
 #include <string.h>
 #include <time.h>
+#include <fnmatch.h>
 
 #include <glib.h>
 #include <stdio.h>
@@ -17,8 +18,12 @@
 #define CANVAS_HEIGHT 150
 #define COL_WIDTH 192
 #define TREE_SPACING 16
+#define TREE_WIDTH 400
+#define TREE_HEIGHT 320
 
-#define _FREE(a) if (a) g_free (a)
+#define FNAME 0
+
+#define MY_FREE(a) if (a) g_free (a)
 #define MY_STRLEN(x) (x?strlen(x):0)
 #define MY_STRDUP(x) (*x?g_strdup(x):NULL)
 
@@ -28,15 +33,18 @@
 #define EXP 3
 #define SEC 4
 
+#define PHONE 1
+#define EMAIL 2
+
 GtkWidget *gnomecard_window;
 GtkWidget *crd_canvas;
 GnomeCanvasItem *test;
 GtkCTree  *crd_tree;
 
 GtkWidget *tb_next, *tb_prev, *tb_first, *tb_last;
-GtkWidget *tb_edit, *tb_save, *tb_del;
+GtkWidget *tb_edit, *tb_find, *tb_save, *tb_del;
 GtkWidget *menu_next, *menu_prev, *menu_first, *menu_last;
-GtkWidget *menu_edit, *menu_save, *menu_del;
+GtkWidget *menu_edit, *menu_find, *menu_save, *menu_del;
 
 GnomeUIInfo *add_menu; 
 
@@ -47,8 +55,12 @@ pix *crd_pix, *ident_pix, *geo_pix, *org_pix, *exp_pix, *sec_pix;
 pix *phone_pix, *email_pix, *addr_pix, *expl_pix, *org_pix;
 
 char *gnomecard_fname;
+char *gnomecard_find_str;
+gboolean gnomecard_find_sens;
+gboolean gnomecard_find_back;
+gint gnomecard_def_data;
 gboolean gnomecard_changed;
-gboolean found;                 /* yeah... pretty messy. (fixme) */
+gboolean gnomecard_found;                 /* yeah... pretty messy. (fixme) */
 
 char *email_type_name[] = 
                  { N_("America On-Line"), N_("Apple Link"), N_("AT&T"),
@@ -89,9 +101,9 @@ GnomeStockPixmapEntry *gnomecard_pentry_new(gchar **xpm_data, gint size)
 void gnomecard_init_stock(void)
 {
 	GnomeStockPixmapEntry *pentry;
-	gchar **xpms[] = { cardnew_xpm, cardedit_xpm, first_xpm, last_xpm,
+	gchar **xpms[] = { cardnew_xpm, cardedit_xpm, first_xpm, last_xpm, cardfind_xpm,
 		                 addr_xpm, phone_xpm, email_xpm, NULL };
-	gchar *names[] = { "New", "Edit", "First", "Last", "Addr", "Phone", "EMail" };
+	gchar *names[] = { "New", "Edit", "First", "Last", "Find", "Addr", "Phone", "EMail" };
 	gchar stockname[22];
 	int i;
 	
@@ -158,6 +170,78 @@ void gnomecard_init_pixes(void)
 	org_pix = pix_new(org_xpm);
 }
 	
+void gnomecard_init_defaults(void)
+{
+	gnomecard_def_data = gnome_config_get_int("/GnomeCard/layout/def_data=0");
+	gnomecard_find_sens = gnome_config_get_bool("/GnomeCard/find/sens=False");
+	gnomecard_find_back = gnome_config_get_bool("/GnomeCard/find/back=False");
+	gnomecard_find_str = gnome_config_get_string("/GnomeCard/find/str=");
+	gnomecard_fname = gnome_config_get_string("/GnomeCard/file/open=");
+}
+
+void gnomecard_set_changed(gboolean val)
+{
+	gnomecard_changed = val;
+	gtk_widget_set_sensitive(tb_save, val);
+	gtk_widget_set_sensitive(menu_save, val);
+}
+
+void gnomecard_scroll_tree(GList *node)
+{
+	GtkCTreeNode *tree_node;
+	
+	tree_node = ((Card *) node->data)->user_data;
+
+	gtk_ctree_moveto(crd_tree, tree_node, 0, 0, 0);
+	gtk_ctree_select(crd_tree, tree_node);
+}
+
+int gnomecard_cmp_fnames(const void *crd1, const void *crd2)
+{
+	return strcmp((* (Card **) crd1)->fname.str, (*(Card **) crd2)->fname.str);
+}
+
+void gnomecard_sort_cards(GtkWidget *w, gpointer data)
+{
+	int (*compar)(const void *, const void *);
+	GList *l;
+	Card **array, *curr;
+	guint i, len, criteria;
+	
+	criteria = (guint) data;
+	curr = curr_crd->data;
+	
+	switch (criteria) {
+	 case FNAME:
+		compar = gnomecard_cmp_fnames;
+		break;
+	}
+	
+	len = g_list_length(crds);
+	array = g_malloc(sizeof(Card *) * len);
+	i = 0;
+	for (l = crds; l; l = l->next)
+		array[i++] = l->data;
+	
+	qsort(array, len, sizeof(Card *), compar);
+	
+	i = 0;
+	gtk_clist_freeze(GTK_CLIST(crd_tree));
+	for (l = crds; l; l = l->next, i++) {
+		if (curr == array[i])
+			curr_crd = l;
+		
+		l->data = array[i];
+		gtk_ctree_move(crd_tree, array[i]->user_data, NULL, NULL);
+	}
+	gtk_clist_thaw(GTK_CLIST(crd_tree));
+	
+	gnomecard_scroll_tree(curr_crd);
+	gnomecard_set_changed(TRUE);
+	
+	g_free(array);
+}
+
 void gnomecard_toggle_card_view(GtkWidget *w, gpointer data)
 {
 	if (GTK_CHECK_MENU_ITEM(w)->active)
@@ -313,8 +397,8 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 			text[0] = _("Comment");
 			text[1] = "";
 			comment = gtk_ctree_insert(crd_tree, expl, NULL, text, TREE_SPACING, 
-																 expl_pix->pixmap,	expl_pix->mask, 
-																 expl_pix->pixmap, expl_pix->mask, FALSE, FALSE);
+						   expl_pix->pixmap,	expl_pix->mask, 
+						   expl_pix->pixmap, expl_pix->mask, FALSE, FALSE);
 			
 			text[0] = "";
 			c = rem = text[1] = g_strdup(crd->comment.str);
@@ -349,30 +433,36 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 	if (crd->phone) {
 		GtkCTreeNode *phone, *phone2;
 		GList *l;
-		int i;
+		int i, len;
 		
 		text[0] = _("Telephone Numbers");
 		text[1] = "";
-		phone = gtk_ctree_insert(crd_tree, parent, NULL, text,
-														 TREE_SPACING, phone_pix->pixmap,
-														 phone_pix->mask, phone_pix->pixmap,
-														 phone_pix->mask, FALSE, FALSE);
+		if (! crd->phone->next)
+			phone = parent;
+		else
+			phone = gtk_ctree_insert(crd_tree, parent, NULL, text,
+															 TREE_SPACING, phone_pix->pixmap,
+															 phone_pix->mask, phone_pix->pixmap,
+															 phone_pix->mask, FALSE, FALSE);
 		
 		for (l = crd->phone; l; l = l->next) {
-			text[0] = "";
-				/*phone_type_name[((CardEMail *)l->data)->type];*/
-			text[1] = ((CardPhone *)l->data)->data;
-			phone2 = gtk_ctree_insert(crd_tree, phone, NULL, text, TREE_SPACING,
-																phone_pix->pixmap, phone_pix->mask, 
-																phone_pix->pixmap, phone_pix->mask, 
-																FALSE, FALSE);
-			text[1] = "";
+			text[0] = ((CardPhone *)l->data)->data;
+			text[1] = malloc(1);
+			*text[1] = 0;
+			len = 1;
 			for (i = 0; i < 13; i++)
-				if (((CardPhone *)l->data)->type & (1 << i)) {
-					text[0] = phone_type_name[i];
-					gtk_ctree_insert(crd_tree, phone2, NULL, text, TREE_SPACING, 
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
+			  if (((CardPhone *)l->data)->type & (1 << i)) {
+				  len += strlen(phone_type_name[i]) + 1;
+				  text[1] = realloc(text[1], len);
+				  snprintf(text[1], len, "%s %s", text[1], 
+					   phone_type_name[i]);
+			  }
+			
+			phone2 = gtk_ctree_insert(crd_tree, phone, NULL, text, TREE_SPACING,
+						  phone_pix->pixmap, phone_pix->mask, 
+						  phone_pix->pixmap, phone_pix->mask,
+						  FALSE, FALSE);
+			free(text[1]);
 		}
 	}
 	
@@ -382,17 +472,22 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 		
 		text[0] = _("E-mail Addresses");
 		text[1] = "";
-		email = gtk_ctree_insert(crd_tree, parent, NULL, text,
-														 TREE_SPACING, email_pix->pixmap,
-														 email_pix->mask, email_pix->pixmap,
-														 email_pix->mask, FALSE, FALSE);
+		
+		if (! crd->email->next)
+			email = parent;
+		else
+			email = gtk_ctree_insert(crd_tree, parent, NULL, text,
+															 TREE_SPACING, email_pix->pixmap,
+															 email_pix->mask, email_pix->pixmap,
+															 email_pix->mask, FALSE, FALSE);
 		
 		for (l = crd->email; l; l = l->next) {
 			text[0] = email_type_name[((CardEMail *)l->data)->type];
 			text[1] = ((CardEMail *)l->data)->data;
 			gtk_ctree_insert(crd_tree, email, NULL, text,
-					 TREE_SPACING, NULL, NULL, NULL, NULL,
-					 FALSE, FALSE);
+											 TREE_SPACING, email_pix->pixmap,
+											 email_pix->mask, email_pix->pixmap,
+											 email_pix->mask, FALSE, FALSE);
 		}
 	}
 	
@@ -403,23 +498,28 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 		
 		text[0] = _("Delivery Addresses");
 		text[1] = "";
-		addr = gtk_ctree_insert(crd_tree, parent, NULL, text, TREE_SPACING, 
-														addr_pix->pixmap, addr_pix->mask, 
-														addr_pix->pixmap, addr_pix->mask, FALSE, FALSE);
+		
+		/* fixme: not handling dellabel case. */
+		
+		if (! crd->deladdr->next)
+			addr = parent;
+		else
+			addr = gtk_ctree_insert(crd_tree, parent, NULL, text, TREE_SPACING, 
+															addr_pix->pixmap, addr_pix->mask, 
+															addr_pix->pixmap, addr_pix->mask, FALSE, FALSE);
 		
 		for (l = crd->deladdr; l; l = l->next) {
 			CardDelAddr *deladdr;
 			
 			text[0] = "";
-			
+			text[1] = NULL;
 			deladdr = ((CardDelAddr *)l->data);
-			text[1] = deladdr->po ? deladdr->po :
-			          deladdr->ext ? deladdr->ext :
-			          deladdr->street ? deladdr->street :
-			          deladdr->city ? deladdr->city :
-			          deladdr->region ? deladdr->region :
-			          deladdr->code ? deladdr->code : 
-			          deladdr->country ? deladdr->country : NULL;
+			
+			for (i = 0; i < DELADDR_MAX; i++)
+				if (deladdr->data[i]) {
+					text[1] = deladdr->data[i];
+					break;
+				}
 			
 			if (text[1]) {
 				addr2 = gtk_ctree_insert(crd_tree, addr, NULL, text, TREE_SPACING,
@@ -427,42 +527,13 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 																 addr_pix->pixmap, addr_pix->mask, 
 																 FALSE, FALSE);
 				text[0] = "";
-				
-				if (deladdr->po && text[1] != deladdr->po) {
-					text[1] = deladdr->po;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->ext && text[1] != deladdr->ext) {
-					text[1] = deladdr->ext;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->street && text[1] != deladdr->street) {
-					text[1] = deladdr->street;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->city && text[1] != deladdr->city) {
-					text[1] = deladdr->city;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->region && text[1] != deladdr->region) {
-					text[1] = deladdr->region;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->code && text[1] != deladdr->code) {
-					text[1] = deladdr->code;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
-				if (deladdr->country && text[1] != deladdr->country) {
-					text[1] = deladdr->country;
-					gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
-													 NULL, NULL, NULL, NULL, FALSE, FALSE);
-				}
+
+				for (i = 0; i < DELADDR_MAX; i++)
+					if (deladdr->data[i] && text[1] != deladdr->data[i]) {
+						text[1] = deladdr->data[i];
+						gtk_ctree_insert(crd_tree, addr2, NULL, text, TREE_SPACING,
+														 NULL, NULL, NULL, NULL, FALSE, FALSE);
+					}
 				
 				text[1] = "";
 				for (i = 0; i < 6; i++)
@@ -476,19 +547,50 @@ void gnomecard_add_card_sections_to_tree(Card *crd)
 	}
 }
 
+void gnomecard_set_node_info(Card *crd)
+{
+	char *text, *tmp;
+	gint len;
+	
+	if ((text = crd->fname.str) == NULL)
+	  text = _("No Formatted Name for this card.");
+	
+	gtk_ctree_set_node_info(crd_tree, crd->user_data, text,
+				TREE_SPACING, crd_pix->pixmap,
+				crd_pix->mask, crd_pix->pixmap,
+				crd_pix->mask, FALSE, FALSE);
+
+	text = malloc(1);
+	*text = 0;
+	len = 1;
+	if (crd->phone && (gnomecard_def_data & PHONE)) {
+	  tmp = ((CardPhone *) (crd->phone->data))->data;
+		len += strlen(tmp) + 1;
+		text = realloc(text, len);
+		snprintf(text, len, "%s %s", text, tmp);
+	}
+	if (crd->email && (gnomecard_def_data & EMAIL)) {
+		tmp = ((CardEMail *) (crd->email->data))->data;
+		len += strlen(tmp) + 1;
+		text = realloc(text, len);
+		snprintf(text, len, "%s %s", text, tmp);
+	}
+
+	gtk_ctree_set_text(crd_tree, crd->user_data, 1, text);
+	
+	g_free(text);
+}
+
 void gnomecard_add_card_to_tree(Card *crd)
 {
 	char *text[2];
 	
-	if ((text[1] = crd->fname.str) == NULL)
-	  text[1] = _("No Formatted Name for this card.");
-	text[0] = "";
+	text[0] = text[1] = "";
 	
 	crd->user_data = gtk_ctree_insert(crd_tree, NULL, NULL, text,
-					TREE_SPACING, crd_pix->pixmap,
-					crd_pix->mask, crd_pix->pixmap,
-					crd_pix->mask, FALSE, FALSE);
+																		0, NULL, NULL, NULL, NULL, FALSE, FALSE);
 	
+	gnomecard_set_node_info(crd);
 	gnomecard_add_card_sections_to_tree(crd);
 }
 
@@ -515,7 +617,7 @@ GtkWidget *my_hbox_entry(GtkWidget *parent, char *label, char *init)
 	w = gtk_label_new(label);
 	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w = my_gtk_entry_new(0, init);
-	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	
 	return w;
 }
@@ -532,7 +634,6 @@ GtkWidget *my_gtk_spin_button_new(GtkAdjustment *adj, gint width)
 	return spin;
 }
 
-
 GtkWidget *my_gtk_vbox_new(void)
 {
 	GtkWidget *vbox;
@@ -586,38 +687,21 @@ void gnomecard_set_edit_del(gboolean state)
 	gtk_widget_set_sensitive(menu_edit, state);
 	gtk_widget_set_sensitive(tb_del, state);
 	gtk_widget_set_sensitive(menu_del, state);
-}
-
-void gnomecard_set_changed(gboolean val)
-{
-	gnomecard_changed = val;
-	gtk_widget_set_sensitive(tb_save, val);
-	gtk_widget_set_sensitive(menu_save, val);
-}
-
-void gnomecard_scroll_tree(GList *node)
-{
-	GtkCTreeNode *tree_node;
-	
-	tree_node = ((Card *) node->data)->user_data;
-
-	gtk_ctree_moveto(crd_tree, tree_node, 0, 0, 0);
-	gtk_ctree_select(crd_tree, tree_node);
+	gtk_widget_set_sensitive(tb_find, state);
+	gtk_widget_set_sensitive(menu_find, state);
 }
 
 /* I'm using collapse and expand to avoid ctree_remove SIGSEGV */
 void gnomecard_update_tree(Card *crd)
 {
 	GtkCTreeNode *node, *tmp;
-	char *text;
 	
-	if ((text = crd->fname.str) == NULL)
-	  text = _("No Formatted Name for this card.");
-	
-	gtk_ctree_set_text(crd_tree, crd->user_data,	1, text);
+	gnomecard_set_node_info(crd);
 	gtk_ctree_collapse_recursive(crd_tree, crd->user_data);
 	
-	node = crd->user_data;
+	/* FIXME: the gtkctree API is broken. This is a workaround.
+	 * GTK_CTREE_NODE_CHILDREN should exist. */
+	node = GTK_CTREE_ROW(&GTK_CTREE_NODE(crd->user_data)->list)->children;
 	while (node) {
 		tmp = GTK_CTREE_NODE_NEXT (node);
 		gtk_ctree_remove(crd_tree, node);
@@ -625,7 +709,6 @@ void gnomecard_update_tree(Card *crd)
 	}
 	
 	gnomecard_add_card_sections_to_tree(crd);
-	gtk_ctree_expand_recursive(crd_tree, crd->user_data);
 }
 
 void gnomecard_update_canvas(Card *crd) 
@@ -674,19 +757,19 @@ void gnomecard_set_curr(GList *node)
 void gnomecard_cmp_nodes(GtkCTree *tree, GList *node, gpointer data)
 {
 	if ((GList *) data == node)
-	  found = TRUE;
+	  gnomecard_found = TRUE;
 }
 
 void gnomecard_tree_selected(GtkCTree *tree, GList *row, gint column)
 {
 	GList *i;
 	
-	found = FALSE;
+	gnomecard_found = FALSE;
 	for (i = crds; i; i = i->next) {
 		gtk_ctree_post_recursive(tree, ((Card *) i->data)->user_data,
 					 GTK_CTREE_FUNC(gnomecard_cmp_nodes), 
 					 row);
-		if (found)
+		if (gnomecard_found)
 		  break;
 	}
 	
@@ -725,12 +808,12 @@ void gnomecard_prop_apply(GtkWidget *widget, int page)
 	ce = (GnomeCardEditor *) gtk_object_get_user_data(GTK_OBJECT(widget));
 	crd = (Card *) ce->l->data;
 	
-	_FREE(crd->fname.str);
-	_FREE(crd->name.family);
-	_FREE(crd->name.given);
-	_FREE(crd->name.additional);
-	_FREE(crd->name.prefix);
-	_FREE(crd->name.suffix);
+	MY_FREE(crd->fname.str);
+	MY_FREE(crd->name.family);
+	MY_FREE(crd->name.given);
+	MY_FREE(crd->name.additional);
+	MY_FREE(crd->name.prefix);
+	MY_FREE(crd->name.suffix);
 	
 	crd->fname.str       = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->fn)));
 	crd->name.family     = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->fam)));
@@ -753,13 +836,13 @@ void gnomecard_prop_apply(GtkWidget *widget, int page)
 	crd->geopos.lon = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ce->gplon));
 	crd->geopos.lat = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ce->gplat));
 	
-	_FREE(crd->title.str);
-	_FREE(crd->role.str);
-	_FREE(crd->org.name);
-	_FREE(crd->org.unit1);
-	_FREE(crd->org.unit2);
-	_FREE(crd->org.unit3);
-	_FREE(crd->org.unit4);
+	MY_FREE(crd->title.str);
+	MY_FREE(crd->role.str);
+	MY_FREE(crd->org.name);
+	MY_FREE(crd->org.unit1);
+	MY_FREE(crd->org.unit2);
+	MY_FREE(crd->org.unit3);
+	MY_FREE(crd->org.unit4);
 
 	crd->title.str = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->title)));
 	crd->role.str  = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->role)));
@@ -769,14 +852,14 @@ void gnomecard_prop_apply(GtkWidget *widget, int page)
 	crd->org.unit3 = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->org3)));
 	crd->org.unit4 = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->org4)));
 	
-	_FREE(crd->comment.str);
-	_FREE(crd->url.str);
+	MY_FREE(crd->comment.str);
+	MY_FREE(crd->url.str);
 	
 	crd->comment.str = gtk_editable_get_chars(GTK_EDITABLE(ce->comment), 
 						      0, gtk_text_get_length(GTK_TEXT(ce->comment)));
 	crd->url.str     = MY_STRDUP(gtk_entry_get_text(GTK_ENTRY(ce->url)));
 	
-        _FREE(crd->key.data);
+        MY_FREE(crd->key.data);
 	
 	crd->key.data = gtk_editable_get_chars(GTK_EDITABLE(ce->key), 
 						   0, gtk_text_get_length(GTK_TEXT(ce->key)));
@@ -839,8 +922,8 @@ void gnomecard_edit(GList *node, int section)
 	gnomecard_set_add(TRUE);
 	
 	box = GNOME_PROPERTY_BOX(gnome_property_box_new());
-	gtk_object_set_user_data(GTK_OBJECT(box), (gpointer) ce);
-	gtk_window_set_wmclass(GTK_WINDOW(box), "gnomecard",
+	gtk_object_set_user_data(GTK_OBJECT(box), ce);
+	gtk_window_set_wmclass(GTK_WINDOW(box), "GnomeCard",
 			       "GnomeCard");
 	gtk_signal_connect(GTK_OBJECT(box), "apply",
 			   (GtkSignalFunc)gnomecard_prop_apply, NULL);
@@ -1126,6 +1209,11 @@ void gnomecard_edit(GList *node, int section)
 
 void gnomecard_cancel(GtkWidget *widget, gpointer data)
 {
+	void *p;
+
+	if ((p = gtk_object_get_user_data(GTK_OBJECT(widget))) != NULL)
+		g_free(p);
+	
 	gtk_widget_destroy(widget);
 }
 
@@ -1185,7 +1273,7 @@ gnomecard_destroy_cards(void)
 
 	if (gnomecard_cards_blocked()) {
 		GtkWidget *w;
-		
+
 		w = gnome_message_box_new("There are cards which are currently being modified.\nFinish any pending modifications and try again.",
 					  GNOME_MESSAGE_BOX_ERROR,
 					  GNOME_STOCK_BUTTON_OK, NULL);
@@ -1278,6 +1366,190 @@ void gnomecard_del_card(GtkWidget *widget, gpointer data)
 	gnomecard_set_changed(TRUE);
 }
 
+int gnomecard_match_pattern(char *pattern, char *str, int sens)
+{
+	char *txt;
+	int found = 0;
+	
+	if (! str)
+		return 0;
+	
+	if (sens)
+		txt = str;
+	else
+		g_strup(txt = g_strdup(str));
+	
+	if (fnmatch(pattern, txt, 0) == 0) {
+		found = 1;
+	}
+	
+	if (! sens)
+		g_free(txt);
+	
+	return found;
+}
+
+void gnomecard_find_card(GtkWidget *w, gpointer data)
+{
+	GnomeCardFind *p;
+	GList *l, *k;
+	Card *crd;
+	char *pattern, *crd_text[15];
+	int i, wrapped, found, sens, back;
+	
+	p = (GnomeCardFind *) data;
+
+	found = 0;
+	wrapped = 0;
+
+	if (GTK_TOGGLE_BUTTON(p->back)->active)
+		gnomecard_find_back = back = 1;
+	else
+		gnomecard_find_back = back = 0;
+	
+	MY_FREE(gnomecard_find_str);
+	gnomecard_find_str = g_strdup(gtk_entry_get_text(GTK_ENTRY(p->entry)));
+	pattern = g_malloc(strlen(gnomecard_find_str) + 3);
+	sprintf(pattern, "*%s*", gnomecard_find_str);
+
+	if (GTK_TOGGLE_BUTTON(p->sens)->active)
+		gnomecard_find_sens = sens = 1;
+	else {
+		gnomecard_find_sens = sens = 0;
+		g_strup(pattern);
+	}
+	
+	l = curr_crd;
+	
+	while (l) {
+		if (wrapped != 1)
+			l = (back)? l->prev : l->next;
+		else
+			wrapped ++;
+
+		if (l) {
+			crd = l->data;
+			crd_text[0] = crd->fname.str;
+			crd_text[1] = crd->name.family;
+			crd_text[2] = crd->name.given;
+			crd_text[3] = crd->name.additional;
+			crd_text[4] = crd->name.prefix;
+			crd_text[5] = crd->name.suffix;
+			crd_text[6] = crd->title.str;
+			crd_text[7] = crd->role.str;
+			crd_text[8] = crd->comment.str;
+			crd_text[9] = crd->url.str;
+			crd_text[10] = crd->org.name;
+			crd_text[11] = crd->org.unit1;
+			crd_text[12] = crd->org.unit2;
+			crd_text[13] = crd->org.unit3;
+			crd_text[14] = crd->org.unit4;
+			
+			for (i = 0; i < 15 && !found; i++)
+				if (gnomecard_match_pattern(pattern, crd_text[i], sens))
+					found = 1;
+
+			for (k = crd->phone; k && !found; k = k->next)
+				if (gnomecard_match_pattern(pattern, 
+																		((CardPhone *) k->data)->data, sens))
+					found = 1;
+			
+			for (k = crd->email; k && !found; k = k->next)
+				if (gnomecard_match_pattern(pattern, 
+																		((CardEMail *) k->data)->data, sens))
+						found = 1;
+
+			for (k = crd->dellabel; k && !found; k = k->next)
+				if (gnomecard_match_pattern(pattern, 
+																		((CardDelLabel *) k->data)->data, sens))
+					found = 1;
+
+			for (k = crd->deladdr; k && !found; k = k->next)
+				for (i = 0; i < DELADDR_MAX; i++)
+					if (gnomecard_match_pattern(pattern, 
+																			((CardDelAddr *) k->data)->data[i], sens))
+						found = 1;
+			
+			if (found) {
+				gnomecard_scroll_tree(l);
+				break;
+			}
+			
+		}	else if (wrapped) {
+			GtkWidget *w;
+			
+			w = gnome_message_box_new(_("No matching record found."),
+																GNOME_MESSAGE_BOX_ERROR,
+																GNOME_STOCK_BUTTON_OK, NULL);
+			GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
+			gnome_dialog_button_connect_object(GNOME_DIALOG(w),	0,
+																				 GTK_SIGNAL_FUNC(gnomecard_cancel),
+																				 GTK_OBJECT(w));	
+			gtk_widget_show(w);
+			break;
+		} else {
+			GtkWidget *w;
+			char msg[128], *str1, *str2;
+			
+			str1 = (back)? _("first") : _("last");
+			str2 = (back)? _("last") : _("first");
+			snprintf(msg, 128, _("Reached %s record.\nContinue from the %s one?"),
+							 str1, str2);
+			w = gnome_message_box_new(msg,
+																GNOME_MESSAGE_BOX_QUESTION,
+																GNOME_STOCK_BUTTON_OK,
+																GNOME_STOCK_BUTTON_CANCEL, NULL);
+			GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
+			gtk_widget_show(w);
+			
+			switch(gnome_dialog_run_modal(GNOME_DIALOG(w))) {
+			 case -1:
+			 case 1:
+				l = NULL;
+				break;
+			 case 0:
+				l = (back)? g_list_last(crds) : crds;
+				wrapped = 1;
+			}
+		}
+	}
+
+	g_free(pattern);
+	
+	gnome_config_set_bool("/GnomeCard/find/sens",  gnomecard_find_sens);
+	gnome_config_set_bool("/GnomeCard/find/back",  gnomecard_find_back);
+	gnome_config_set_string("/GnomeCard/find/str",  gnomecard_find_str);
+}
+
+void gnomecard_find_card_call(GtkWidget *widget, gpointer data)
+{
+	GtkWidget *w, *hbox, *check;
+	GnomeCardFind *p;
+
+	p = g_malloc(sizeof(GnomeCardPhone));
+	w = gnome_dialog_new(_("Find Card"), _("Find"),
+											 GNOME_STOCK_BUTTON_CLOSE, NULL);
+	gtk_object_set_user_data(GTK_OBJECT(w), p);
+	gnome_dialog_button_connect(GNOME_DIALOG(w), 0,
+															GTK_SIGNAL_FUNC(gnomecard_find_card),	p);
+	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	1,
+																		 GTK_SIGNAL_FUNC(gnomecard_cancel),
+																		 GTK_OBJECT(w));	
+																		 
+	p->entry = my_hbox_entry(GNOME_DIALOG(w)->vbox, _("Find:"), gnomecard_find_str);
+	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox), hbox, FALSE, FALSE, 0);
+	p->sens = check = gtk_check_button_new_with_label(_("Case Sensitive"));
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check), gnomecard_find_sens);
+	gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 0);
+	p->back = check = gtk_check_button_new_with_label(_("Find Backwards"));
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check), gnomecard_find_back);
+	gtk_box_pack_end(GTK_BOX(hbox), check, FALSE, FALSE, 0);
+	
+	gtk_widget_show_all(GNOME_DIALOG(w)->vbox);
+	gtk_widget_show(w);
+}
+
 void gnomecard_add_email(GtkWidget *widget, gpointer data)
 {
 	GnomeCardEMail *e;
@@ -1321,6 +1593,7 @@ void gnomecard_add_email_call(GtkWidget *widget, gpointer data)
 	e = g_malloc(sizeof(GnomeCardEMail));
 	e->l = curr_crd;
 	w = gnome_dialog_new(title, "Add", GNOME_STOCK_BUTTON_CLOSE, NULL);
+	gtk_object_set_user_data(GTK_OBJECT(w), e);
 	gnome_dialog_button_connect(GNOME_DIALOG(w), 0,
 															GTK_SIGNAL_FUNC(gnomecard_add_email),	e);
 	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	1,
@@ -1399,6 +1672,7 @@ void gnomecard_add_phone_call(GtkWidget *widget, gpointer data)
 	p = g_malloc(sizeof(GnomeCardPhone));
 	p->l = curr_crd;
 	w = gnome_dialog_new(title, "Add", GNOME_STOCK_BUTTON_CLOSE, NULL);
+	gtk_object_set_user_data(GTK_OBJECT(w), p);
 	gnome_dialog_button_connect(GNOME_DIALOG(w), 0,
 															GTK_SIGNAL_FUNC(gnomecard_add_phone),	p);
 	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	1,
@@ -1454,14 +1728,11 @@ void gnomecard_add_deladdr(GtkWidget *widget, gpointer data)
 		return;
 	
 	addr = g_malloc(sizeof(CardDelAddr));
-	addr->po = MY_STRDUP(text[0]);
-	addr->ext = MY_STRDUP(text[1]);
-	addr->street = MY_STRDUP(text[2]);
-	addr->city = MY_STRDUP(text[3]);
-	addr->region = MY_STRDUP(text[4]);
-	addr->code = MY_STRDUP(text[5]);
-	addr->country = MY_STRDUP(text[6]);
-  addr->prop = empty_CardProperty();
+	
+	for (i = 0; i < DELADDR_MAX; i++)
+		addr->data[i] = MY_STRDUP(text[i]);
+	
+	addr->prop = empty_CardProperty();
 	addr->prop.used = TRUE;
 
 	addr->type = 0;
@@ -1472,7 +1743,7 @@ void gnomecard_add_deladdr(GtkWidget *widget, gpointer data)
 	((Card *) p->l->data)->deladdr = g_list_append(((Card *) p->l->data)->deladdr, addr);
 	gnomecard_update_tree(p->l->data);
 
-	for (i = 0; i < 7; i++)
+	for (i = 0; i < DELADDR_MAX; i++)
 		gtk_editable_delete_text(GTK_EDITABLE(p->data[i]), 0, strlen(text[i]));
 	gnomecard_set_changed(TRUE);
 }
@@ -1495,6 +1766,7 @@ void gnomecard_add_deladdr_call(GtkWidget *widget, gpointer data)
 	p = g_malloc(sizeof(GnomeCardDelAddr));
 	p->l = curr_crd;
 	w = gnome_dialog_new(title, "Add", GNOME_STOCK_BUTTON_CLOSE, NULL);
+	gtk_object_set_user_data(GTK_OBJECT(w), p);
 	gnome_dialog_button_connect(GNOME_DIALOG(w), 0,
 															GTK_SIGNAL_FUNC(gnomecard_add_deladdr),	p);
 	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	1,
@@ -1515,7 +1787,7 @@ void gnomecard_add_deladdr_call(GtkWidget *widget, gpointer data)
 		gtk_box_pack_start(GTK_BOX(vbox), p->type[i], FALSE, FALSE, 0);
 	}
 	
-	table = my_gtk_table_new(2, 7);
+	table = my_gtk_table_new(2, DELADDR_MAX);
 	gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
 	
 	label = gtk_label_new("Post Office:");
@@ -1565,9 +1837,40 @@ void gnomecard_add_deladdr_call(GtkWidget *widget, gpointer data)
 	gtk_widget_show(w);
 }
 
+void gnomecard_add_dellabel(GtkWidget *widget, gpointer data)
+{
+	GnomeCardDelLabel *p;
+	CardDelLabel *addr;
+	char *text;
+	int i;
+	
+	p = (GnomeCardDelLabel *) data;
+	
+	text = gtk_editable_get_chars(GTK_EDITABLE(p->data), 0,
+				      gtk_text_get_length(GTK_TEXT(p->data)));
+	if (*text == 0)
+	  return;
+	
+	addr = g_malloc(sizeof(CardDelLabel));
+	addr->data = MY_STRDUP(text);
+	addr->prop = empty_CardProperty();
+	addr->prop.encod = ENC_QUOTED_PRINTABLE;
+	addr->prop.used = TRUE;
+
+	addr->type = 0;
+	for (i = 0; i < 6; i++)
+	  if (GTK_TOGGLE_BUTTON(p->type[i])->active)
+	    addr->type |= (int) gtk_object_get_user_data(GTK_OBJECT(p->type[i]));
+	
+	((Card *) p->l->data)->dellabel = g_list_append(((Card *) p->l->data)->dellabel, addr);
+	gnomecard_update_tree(p->l->data);
+
+	gtk_editable_delete_text(GTK_EDITABLE(p->data), 0, strlen(text));
+	gnomecard_set_changed(TRUE);
+}
+
 void gnomecard_add_dellabel_call(GtkWidget *widget, gpointer data)
 {
-	g_warning ("gnomecard_add_dellabel_call.");
 }
 
 void gnomecard_edit_card(GtkWidget *widget, gpointer data)
@@ -1598,34 +1901,49 @@ void gnomecard_last_card(GtkWidget *widget, gpointer data)
 	gnomecard_scroll_tree(g_list_last(crds));
 }
 
-void gnomecard_open_call(GtkWidget *widget, gpointer data)
+gboolean gnomecard_open_file(char *fname)
 {
-	char *fname;
 	GtkWidget *w;
 	GList *c;
-	
-	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget));
-	
+
 	if (!(c = card_load(NULL, fname))) {
-		w = gnome_message_box_new(_("Wrong file format."),
+		char *tmp;
+		
+		tmp = g_malloc(strlen(_("Wrong file format.")) + strlen(fname) + 3);
+		sprintf(tmp, "%s: %s", fname, _("Wrong file format."));
+		w = gnome_message_box_new(tmp,
 					  GNOME_MESSAGE_BOX_ERROR,
 					  GNOME_STOCK_BUTTON_OK, NULL);
 		GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
 		gtk_widget_show(w);
-	} else {
-		if (gnomecard_destroy_cards()) {
-			crds = c;
-			
-			gtk_clist_freeze(GTK_CLIST(crd_tree));
-			for (c = crds; c; c = c->next)
-			  gnomecard_add_card_to_tree((Card *) c->data);
-			gtk_clist_thaw(GTK_CLIST(crd_tree));
-			
-			g_free(gnomecard_fname);
-			gnomecard_fname = g_strdup(fname);
-			gnomecard_scroll_tree(g_list_first(crds));
-			gtk_widget_destroy(widget);
-		}
+		g_free(tmp);
+		
+		return FALSE;
+	} else if (gnomecard_destroy_cards()) {
+		crds = c;
+		
+		gtk_clist_freeze(GTK_CLIST(crd_tree));
+		for (c = crds; c; c = c->next)
+			gnomecard_add_card_to_tree((Card *) c->data);
+		gtk_clist_thaw(GTK_CLIST(crd_tree));
+		
+		gnomecard_scroll_tree(g_list_first(crds));
+	}
+	
+	return TRUE;
+}
+
+void gnomecard_open_call(GtkWidget *widget, gpointer data)
+{
+	char *fname;
+	
+	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget));
+	
+	if (gnomecard_open_file(fname)) {
+		g_free(gnomecard_fname);
+		gnomecard_fname = g_strdup(fname);
+		gtk_widget_destroy(widget);
+		gnome_config_set_string("/GnomeCard/file/open",  gnomecard_fname);
 	}
 }
 
@@ -1644,9 +1962,72 @@ void gnomecard_open(GtkWidget *widget, gpointer data)
 	gtk_widget_show(fsel);
 }
 
+void gnomecard_setup_apply(GtkWidget *widget, int page)
+{
+	GnomeCardSetup *setup;
+	GList *i;
+	int old_def_data;
+
+	if (page != -1)
+	  return;             /* ignore partial applies */
+	
+	setup = (GnomeCardSetup *) gtk_object_get_user_data(GTK_OBJECT(widget));
+	
+	old_def_data = gnomecard_def_data;
+	gnomecard_def_data = 0;
+	
+	if (GTK_TOGGLE_BUTTON(setup->def_phone)->active)
+		gnomecard_def_data |= PHONE;
+	
+	if (GTK_TOGGLE_BUTTON(setup->def_email)->active)
+		gnomecard_def_data |= EMAIL;
+	
+	if (gnomecard_def_data != old_def_data)
+		for (i = crds; i; i = i->next)
+			gnomecard_set_node_info((Card *) i->data);
+
+	gnome_config_set_int("/GnomeCard/layout/def_data",  gnomecard_def_data);
+}
+			
 void gnomecard_setup(GtkWidget *widget, gpointer data)
 {
-	g_warning ("Setup card.");
+	GnomePropertyBox *box;
+	GnomeCardSetup *setup;
+	GtkWidget *vbox, *vbox2, *frame;
+	GtkWidget *label, *check;
+	
+	setup = g_malloc(sizeof(GnomeCardSetup));
+	box = GNOME_PROPERTY_BOX(gnome_property_box_new());
+	gtk_object_set_user_data(GTK_OBJECT(box), setup);
+	gtk_window_set_wmclass(GTK_WINDOW(box), "GnomeCard",
+			       "GnomeCard");
+	gtk_signal_connect(GTK_OBJECT(box), "apply",
+			   (GtkSignalFunc)gnomecard_setup_apply, NULL);
+
+	vbox = my_gtk_vbox_new();
+	label = gtk_label_new(_("Layout"));
+	gtk_notebook_append_page(GTK_NOTEBOOK(box->notebook), vbox, label);
+	
+	frame = gtk_frame_new(_("Default data"));
+	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+	
+	vbox2 = my_gtk_vbox_new();
+	gtk_container_add(GTK_CONTAINER(frame), vbox2);
+	/* falta conectar con el apply... checa my_connect. */
+	check = setup->def_phone = gtk_check_button_new_with_label("Phone");
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check), gnomecard_def_data & PHONE);
+ 	gtk_signal_connect_object(GTK_OBJECT(check), "clicked",
+				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
+				  GTK_OBJECT(box));
+	gtk_box_pack_start(GTK_BOX(vbox2), check, FALSE, FALSE, 0);
+	check = setup->def_email = gtk_check_button_new_with_label("E-mail");
+ 	gtk_signal_connect_object(GTK_OBJECT(check), "clicked",
+				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
+				  GTK_OBJECT(box));
+	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check), gnomecard_def_data & EMAIL);
+	gtk_box_pack_start(GTK_BOX(vbox2), check, FALSE, FALSE, 0);
+	
+	gtk_widget_show_all(GTK_WIDGET(box));
 }
 
 void gnomecard_quit(GtkWidget *widget, gpointer data)
@@ -1671,13 +2052,13 @@ void gnomecard_spawn_new(GtkWidget *widget, gpointer data)
 {
 	GtkWidget *w;
 	int pid;
-	char *text[] = { "gnomecard", NULL };
+	char *text[] = { "GnomeCard", NULL };
 	
 	pid = fork();
 	if (pid == 0) { /* child */
 /* gnomecard: error in loading shared libraries
  * /gnome/lib/libgnome.so.0: undefined symbol: stat */
-		if (execvp("gnomecard", text) == -1) { 
+		if (execvp("GnomeCard", text) == -1) { 
 			w = gnome_message_box_new(_("A new Gnomecard could not be spawned. Maybe it is not in your path."),
 																GNOME_MESSAGE_BOX_ERROR,
 																GNOME_STOCK_BUTTON_OK, NULL);
@@ -1691,15 +2072,15 @@ void gnomecard_spawn_new(GtkWidget *widget, gpointer data)
 void
 gnomecard_about(GtkWidget *widget, gpointer data)
 {
-        GtkWidget *about;
+	GtkWidget *about;
 	const gchar *authors[] = { "arturo@nuclecu.unam.mx", NULL };
-
-        about = gnome_about_new (_("GnomeCard"), NULL,
-				 "(C) 1997-1998 the Free Software Fundation",
-				 authors,
-				 _("Electronic Business Card Manager"),
-				 NULL);
-        gtk_widget_show (about);
+	
+	about = gnome_about_new (_("GnomeCard"), NULL,
+													 "(C) 1997-1998 the Free Software Fundation",
+													 authors,
+													 _("Electronic Business Card Manager"),
+													 NULL);
+	gtk_widget_show (about);
 }
 
 GnomeUIInfo filemenu[] = {
@@ -1746,6 +2127,10 @@ GnomeUIInfo cardmenu[] = {
 	 gnomecard_edit_card, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, "GnomeCardEditMenu", 0, 0, NULL},
 	
+	{GNOME_APP_UI_ITEM, N_("Delete"), N_("Delete card"),
+	 gnomecard_del_card, NULL, NULL,
+	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL},
+	
 	{GNOME_APP_UI_SEPARATOR},
 	
 	{GNOME_APP_UI_ITEM, N_("First"), N_("First card"),
@@ -1766,9 +2151,9 @@ GnomeUIInfo cardmenu[] = {
 	
 	{GNOME_APP_UI_SEPARATOR},
 	
-	{GNOME_APP_UI_ITEM, N_("Delete"), N_("Delete card"),
-	 gnomecard_del_card, NULL, NULL,
-	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL},
+	{GNOME_APP_UI_ITEM, N_("Find"), N_("Search card"),
+	 gnomecard_find_card_call, NULL, NULL,
+	GNOME_APP_PIXMAP_STOCK, "GnomeCardFindMenu", 0, 0, NULL},
 	
 	{GNOME_APP_UI_ENDOFINFO}
 };
@@ -1826,8 +2211,8 @@ GnomeUIInfo mainmenu[] = {
 	{GNOME_APP_UI_SUBTREE, N_("Add"), NULL, addmenu, NULL, NULL,
 	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
 
-	{GNOME_APP_UI_SUBTREE, N_("View"), NULL, viewmenu, NULL, NULL,
-	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
+/*	{GNOME_APP_UI_SUBTREE, N_("View"), NULL, viewmenu, NULL, NULL,
+	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},*/
 
 	{GNOME_APP_UI_JUSTIFY_RIGHT},
 		
@@ -1860,6 +2245,10 @@ GnomeUIInfo toolbar[] = {
 	 gnomecard_edit_card, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, "GnomeCardEdit", 0, 0, NULL},
 	
+	{GNOME_APP_UI_ITEM, N_("Del"), N_("Delete card"),
+	 gnomecard_del_card, NULL, NULL,
+	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CLOSE, 0, 0, NULL},
+	
 	{GNOME_APP_UI_SEPARATOR},
 	
 	{GNOME_APP_UI_ITEM, N_("First"), N_("First card"),
@@ -1880,27 +2269,24 @@ GnomeUIInfo toolbar[] = {
 	
 	{GNOME_APP_UI_SEPARATOR},
 	
-	{GNOME_APP_UI_ITEM, N_("Del"), N_("Delete card"),
-	 gnomecard_del_card, NULL, NULL,
-	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_CLOSE, 0, 0, NULL},
+	{GNOME_APP_UI_ITEM, N_("Find"), N_("Search card"),
+	 gnomecard_find_card_call, NULL, NULL,
+	GNOME_APP_PIXMAP_STOCK, "GnomeCardFind", 0, 0, NULL},
 	
 	{GNOME_APP_UI_ENDOFINFO}
 };
 
-int main (int argc, char *argv[])
+void gnomecard_init(void)
 {
 	GnomeCanvasGroup *root;
 	GtkWidget *canvas, *align, *hpaned;
-	char *titles[] = { N_("Field"), N_("Value")};
+	char *titles[] = { "", ""};
 
-	gnome_init("gnomecard", NULL, argc, argv, 0, NULL);
-        gdk_imlib_init ();
 	gnomecard_init_stock();
 	gnomecard_init_pixes();
-	textdomain(PACKAGE);
 	
 	gnomecard_window = gnome_app_new("GnomeCard", "GnomeCard");
-	gtk_window_set_wmclass(GTK_WINDOW(gnomecard_window), "gnomecard",
+	gtk_window_set_wmclass(GTK_WINDOW(gnomecard_window), "GnomeCard",
 			       "GnomeCard");
 
 	gtk_widget_show(gnomecard_window);
@@ -1908,7 +2294,9 @@ int main (int argc, char *argv[])
 			   GTK_SIGNAL_FUNC(gnomecard_delete), NULL);
 	
 	hpaned = gtk_hpaned_new();
-
+	gtk_paned_handle_size(GTK_PANED(hpaned), GNOME_PAD_SMALL * 2);
+	gtk_paned_gutter_size(GTK_PANED(hpaned), GNOME_PAD_SMALL * 2 + GNOME_PAD_SMALL);
+	
 	gnome_app_set_contents(GNOME_APP(gnomecard_window), hpaned);
 	gnome_app_create_menus(GNOME_APP(gnomecard_window), mainmenu);
 	gnome_app_create_toolbar(GNOME_APP(gnomecard_window), toolbar);
@@ -1920,14 +2308,18 @@ int main (int argc, char *argv[])
 			   GTK_SIGNAL_FUNC(gnomecard_tree_selected), NULL);
 	gtk_ctree_set_line_style (crd_tree, GTK_CTREE_LINES_SOLID);
 	gtk_ctree_set_reorderable (crd_tree, FALSE);
-	gtk_clist_column_titles_passive(GTK_CLIST(crd_tree));
+	gtk_clist_column_title_active(GTK_CLIST(crd_tree), 0);
+	gtk_signal_connect(GTK_OBJECT(GTK_CLIST(crd_tree)->column->button),
+										 "clicked", GTK_SIGNAL_FUNC(gnomecard_sort_cards),
+										 (gpointer) FNAME);
+	gtk_clist_column_title_passive(GTK_CLIST(crd_tree), 1);
 	gtk_clist_set_policy(GTK_CLIST(crd_tree),
 			     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_usize (GTK_WIDGET(crd_tree), 0, 200);
+	gtk_widget_set_usize (GTK_WIDGET(crd_tree), TREE_WIDTH, TREE_HEIGHT);
 	gtk_paned_add1(GTK_PANED(hpaned), GTK_WIDGET(crd_tree));
 	gtk_widget_show(GTK_WIDGET(crd_tree));
 
-	align = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
+	align = gtk_alignment_new(0.5, 0.5, 0.5, 0.5);
 	gtk_paned_add2(GTK_PANED(hpaned), align);
 	gtk_widget_show(align);
 
@@ -1967,26 +2359,40 @@ int main (int argc, char *argv[])
 	
 	tb_save = toolbar[2].widget;
 	tb_edit = toolbar[5].widget;
-	tb_first = toolbar[7].widget;
-	tb_prev = toolbar[8].widget;
-	tb_next = toolbar[9].widget;
-	tb_last = toolbar[10].widget;
-	tb_del  = toolbar[12].widget;
+	tb_del  = toolbar[6].widget;
+	tb_first = toolbar[8].widget;
+	tb_prev = toolbar[9].widget;
+	tb_next = toolbar[10].widget;
+	tb_last = toolbar[11].widget;
+	tb_find = toolbar[13].widget;
 	
 	menu_save = filemenu[4].widget;
 	menu_edit = cardmenu[1].widget;
-	menu_first = cardmenu[3].widget;
-	menu_prev = cardmenu[4].widget;
-	menu_next = cardmenu[5].widget;
-	menu_last = cardmenu[6].widget;
-	menu_del  = cardmenu[8].widget;
+	menu_del  = cardmenu[2].widget;
+	menu_first = cardmenu[4].widget;
+	menu_prev = cardmenu[5].widget;
+	menu_next = cardmenu[6].widget;
+	menu_last = cardmenu[7].widget;
+	menu_find = cardmenu[9].widget;
 
 	add_menu = addmenu;
+	gnomecard_def_data = PHONE;
 	
-	gnomecard_fname = g_strdup("untitled.gcrd");
+	gnomecard_init_defaults();
 	gnomecard_set_changed(FALSE);
-	gnomecard_set_curr(NULL);
 	
+	if (*gnomecard_fname)
+		gnomecard_open_file(gnomecard_fname);
+	else
+		gnomecard_set_curr(NULL);
+}
+
+int main (int argc, char *argv[])
+{
+	textdomain(PACKAGE);
+	gnome_init("GnomeCard", NULL, argc, argv, 0, NULL);
+	gnomecard_init();
+
 	gtk_main();
 	return 0;
 }
