@@ -65,13 +65,13 @@ void
 about_cb (GtkWidget *widget, void *data)
 {
 	GtkWidget *about;
-	gchar *authors[] = {
+	const gchar *authors[] = {
 		"Radek Doulik (doulik@karlin.mff.cuni.cz)",
 		NULL
 	};
 	
 	about = gnome_about_new ( _("GTop"), VERSION,
-				  /* copyrigth notice */
+				  /* copyright notice */
 				  _("(C) 1998 the Free Software Foundation"),
 				  authors,
 				  /* another comments */
@@ -157,7 +157,7 @@ GnomeUIInfo systemMenu [] = {
 
 GnomeUIInfo fileMenu [] = {
 	{GNOME_APP_UI_ITEM, N_("Properties..."), NULL, gtop_properties, NULL, NULL,
-	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, NULL, 0, NULL},
+	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
 	GNOMEUIINFO_SEPARATOR,
 	{GNOME_APP_UI_ITEM, N_("Exit"), NULL, delete_event_handler, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', 0, NULL},
@@ -168,7 +168,9 @@ GnomeUIInfo mainmenu [] = {
 	GNOMEUIINFO_SUBTREE (N_("File"), fileMenu),
 	GNOMEUIINFO_SUBTREE (N_("View"), procViewMenu),
 	GNOMEUIINFO_SUBTREE (N_("Memory"), memUsageMenu),
+#if 0
 	GNOMEUIINFO_SUBTREE (N_("System"), systemMenu),
+#endif
 	GNOMEUIINFO_SUBTREE (N_("Help"), helpMenu),
 	GNOMEUIINFO_END
 };
@@ -252,14 +254,14 @@ main (int argc, char *argv[])
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
+	gnome_init ("gtop", NULL, argc, argv, 0, NULL);
+	
 	/* session management init */
-        client = gnome_client_new_default ();
+        client = gnome_master_client ();
 
         gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
                             GTK_SIGNAL_FUNC (save_state), NULL);
 
-	gnome_init ("gtop", NULL, argc, argv, 0, NULL);
-	
 	session_id = gnome_client_get_id (client);
 
 	/* prepare gtop environment */
