@@ -296,7 +296,7 @@ procview_start_stop (gint start)
 		run_tag = gtk_timeout_add (3000, procview_update, NULL);
 	}
 
-	gnome_stock_pixmap_widget_set_icon ((GnomeStockPixmapWidget *) (TBC (RUN)->icon),
+	gnome_stock_pixmap_widget_set_icon ((GnomeStock *) (TBC (RUN)->icon),
 					    (start) ? GNOME_STOCK_PIXMAP_TIMER :
 					    GNOME_STOCK_PIXMAP_TIMER_STOP);
 
